﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Threading;
using System.Windows.Forms;
using PS4000aBlockCaptureGui; 

namespace PS4824_BasicSample
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }
        
        // GLOBAL VARIABLES
        uint status = 0;
        short handle = 0;
        uint _timebase = 8;
        public const int BUFFER_SIZE = 1024;
        int timeInterval;
        bool _ready = false;
        ushort[] inputRanges = { 10, 20, 50, 100, 200, 500, 1000, 2000, 5000, 10000, 20000, 50000 };
        /////////////////////////
        

        void BlockCallback(short handle, short status, IntPtr pVoid)
        {
            _ready = true;  // flag to say done reading data
        }

        int adc_to_mv(int raw, Imports.Range range)
        {
            return (raw * inputRanges[(int)(range)]) / Imports.MaxValue;
        }

        short mv_to_adc(int mv, Imports.Range range)
        {
            return (short)((mv * Imports.MaxValue) / inputRanges[(int)(range)]);
        }

        private void run()
        {
            status = Imports.OpenUnit(out handle, null);
            if (handle == 0) 
            {
                MessageBox.Show("The Device code not open\n error code : " + status.ToString(), "Device Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                this.Close();
            }
                        
            if (status != 0) //if not usb3 will need to set up systrem to get  the opwer from usb 2.0
            {
                MessageBox.Show("Using USB power", "Under Powered", MessageBoxButtons.OK, MessageBoxIcon.Information);
                status = Imports.ps4000aChangePowerSource(handle, status);
            }

            status = Imports.SetChannel(handle, Imports.Channel.CHANNEL_A, 1, (short)Imports.coupling.DC, Imports.Range.Range_10V, 0);                          // -1 is not a correct range will cause an error
            if (status != 0)
            {
                MessageBox.Show("Error setting channel Error Code : " + status.ToString(), "Error Setting Channel", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
            int maxsamples;
            _timebase = 10000;
            while (Imports.GetTimebase(handle, _timebase, BUFFER_SIZE, out timeInterval, out maxsamples, 0) != 0)
            {
                _timebase++;
            }
                                    
            // ADVANCED TRIGERRING SETTINGS
            Imports.TriggerConditions[] conditions = new Imports.TriggerConditions[1];
            conditions[0].Source = Imports.Channel.CHANNEL_A;
            conditions[0].Condition = Imports.TriggerState.DontCare;
            status = Imports.SetTriggerChannelConditions(handle, conditions, 1, Imports.CONDITIONS_INFO.Add);

            Imports.TriggerDirections[] directions = new Imports.TriggerDirections[1];
            directions[0].Direction = Imports.ThresholdDirection.Exit;
            directions[0].Source = Imports.Channel.CHANNEL_A;
            status = Imports.SetTriggerChannelDirections(handle, directions, (short)1);

            Imports.TriggerChannelProperties[] properties = new Imports.TriggerChannelProperties[1];
            properties[0].Channel = Imports.Channel.CHANNEL_A;
            properties[0].ThresholdMinor = mv_to_adc(200, Imports.Range.Range_10V); //numeric value 655
            properties[0].ThresholdMajor = mv_to_adc(600, Imports.Range.Range_10V); //numeric value 1966
            properties[0].ThresholdMode = Imports.ThresholdMode.Window;
            properties[0].HysteresisMajor = (ushort)mv_to_adc(100, Imports.Range.Range_10V); //numeric value 327
            properties[0].HysteresisMinor = (ushort)mv_to_adc(100, Imports.Range.Range_10V); //numeric value 327
            status = Imports.SetTriggerChannelProperties(handle, properties, (short)1, (short)0, 0);
            ////////////////////////////////////////////////////////////////////////////////////////////

            // SIMPLE TRIGGER SETTINGS (IF ADVANCED TRIGGERING IS USED, YOU MUST COMMENT OUT SetSimpleTrigger CALL.)
            //status = Imports.SetSimpleTrigger(handle, 1,Imports.Channel.CHANNEL_A, mv_to_adc(400, Imports.Range.Range_10V), Imports.ThresholdDirection.Above, 0,0);
            /////////////////////////////////////////////////////////////////////////////////
            
            Imports.ps4000aBlockReady _callbackDelegate;
            uint sampleCount = BUFFER_SIZE;
            PinnedArray<short> minPinned;
            PinnedArray<short> maxPinned;
            int timeIndisposed;
            short[] minBuffers = new short[sampleCount];
            short[] maxBuffers = new short[sampleCount];
            minPinned = new PinnedArray<short>(minBuffers);
            maxPinned = new PinnedArray<short>(maxBuffers);

            status = Imports.SetDataBuffers(handle, (Imports.Channel)0, maxBuffers, minBuffers, (int)sampleCount, 0, Imports.DownSamplingMode.None);
            
            _ready = false;
            _callbackDelegate = BlockCallback;

            status = Imports.RunBlock(handle, 0, (int)sampleCount, _timebase, out timeIndisposed, 0, _callbackDelegate, IntPtr.Zero);

            if (status != 0)
            {
                MessageBox.Show("Error running block :" + status.ToString(), "Error runblock", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            while (!_ready)  {Thread.Sleep(0);}

            if (_ready)
            {
                short overflow;
                Imports.GetValues(handle, 0, ref sampleCount, 1, Imports.DownSamplingMode.None, 0, out overflow);
                for (int i = 0; i < sampleCount; i++)
                {
                    double time = timeInterval * i;
                    int value = adc_to_mv(maxPinned.Target[i], Imports.Range.Range_10V);
                    richTextBox1.AppendText(value.ToString() + Environment.NewLine);
                }

            }
            else
            {
                MessageBox.Show("data collection aborted", "Data Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                this.Close();
            }

            status = Imports.CloseUnit(handle);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            richTextBox1.Clear();
            run();            
        }
    }
}
